/*
 * Copyright (C) 2003 Flameeyes <dgp85@users.sourceforge.net>
 */

#include "kvba.h"
#include "kvbapref.h"
#include "kvbaaddfavorites.h"
#include "kvbaeditfavorites.h"
#include "kvbajoyconf.h"

#include <iostream>

#include <qdir.h>
#include <qstringlist.h>
#include <qtabwidget.h>
#include <qprocess.h>
#include <qlineedit.h>
#include <qlayout.h>

#include <klocale.h>
#include <kconfig.h>
#include <kfiletreeview.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klistview.h>
#include <klistbox.h>
#include <kaction.h>
#include <ktextbrowser.h>
#include <kurlrequester.h>

kvba *Kvba;

kvba::kvba(QWidget *parent, const char* name)
    : KVBAMainBase( parent, name )
{

	allDirs = new KFileTreeView( tab, "allDirs" );
	allDirs->addColumn( tr2i18n( "Directory" ) );
	allDirs->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 1, 0, allDirs->sizePolicy().hasHeightForWidth() ) );
	allDirs->setResizePolicy( QListView::AutoOneFit );
	allDirs->setRootIsDecorated( true );
	
	KFileTreeBranch *home = allDirs->addBranch(QDir::home().path(), "Home", KGlobal::iconLoader()->loadIcon("folder_home", KIcon::Small));
	allDirs->setDirOnlyMode(home, true);
	root = allDirs->addBranch("file:/", "Root", KGlobal::iconLoader()->loadIcon("folder_red", KIcon::Small));
	allDirs->setDirOnlyMode(root, true);
	
	KGlobal::config()->setGroup("general");
		QListViewItem *tvi = allDirs->findItem(root, KGlobal::config()->readEntry("startdir"));
		allDirs->setSelected(tvi, true);
	
	tabLayout->insertWidget( 0, allDirs );
	connect( allDirs, SIGNAL( selectionChanged(QListViewItem*) ), this, SLOT( dirSelected(QListViewItem*) ) );

	vba = new QProcess (this);
	connect(vba, SIGNAL(readyReadStdout()), this, SLOT(vbaReadyStdout()));
	connect(vba, SIGNAL(readyReadStderr()), this, SLOT(vbaReadyStderr()));
		
	KGlobal::config()->setGroup("favorites");
		QMap<QString, QString> entries = KGlobal::config()->entryMap("favorites");
		for(QMap<QString, QString>::iterator it = entries.begin(); it != entries.end(); it++)
		{
			QStringList fav = QStringList::split(',', *it, true);
	
			QListViewItem *el = new QListViewItem(favorites);
			int i = 0;
			for(QStringList::iterator it2 = fav.begin(); it2 != fav.end(); it2++, i++)
				el->setText(i, *it2);
			favorites->insertItem( el );
		}
}

kvba::~kvba()
{
	delete vba;
	delete allDirs;
}

void kvba::dirSelected(QListViewItem *i)
{
	allRoms->clear();
	
	allRoms->insertStringList(
		QDir(
			allDirs->currentURL().path(),
			"*.zip;*.gba;*.gbc;*.gb",
			QDir::Name
			).entryList()
		);
}

void kvba::romDoubleClicked(QListBoxItem *i)
{
	vbaOutput->clear();
	vba->clearArguments();
	KGlobal::config()->setGroup("general");
		QListViewItem *tvi = allDirs->findItem(root, KGlobal::config()->readEntry("startdir"));
		allDirs->setSelected(tvi, true);
		vba->addArgument(KGlobal::config()->readEntry("exepath"));
		vbaPrepareArguments();
		vba->addArgument(allDirs->currentURL().path() + i->text());
		vba->start();
	
	romsView->setCurrentPage(2);
}

void kvba::vbaReadyStdout()
{
	vbaOutput->append(vba->readLineStdout());
}

void kvba::vbaReadyStderr()
{
	vbaOutput->append("<font color='#FF0000'>" + vba->readLineStderr() + "</font>");
}

void kvba::prefActivated()
{
	kvbapref *pref = new kvbapref(this, "preferences");
	pref->exec();
	delete pref;
}

void kvba::vbaPrepareArguments()
{
	KGlobal::config()->setGroup("general");

	int vbaVersion = KGlobal::config()->readNumEntry( "vbaversion" );

	if ( !KGlobal::config()->readEntry( "biosfile", "" ).isEmpty() )
    	vba->addArgument("--bios=" + KGlobal::config()->readEntry( "biosfile", "" ));

	if ( !KGlobal::config()->readEntry( "cfgfile", "" ).isEmpty() )
		vba->addArgument("--config=" + KGlobal::config()->readEntry( "cfgfile", "" ));
	
	KGlobal::config()->setGroup("features");
	if ( KGlobal::config()->readBoolEntry( "rtc", true ) )
		vba->addArgument("--rtc");
	else
		vba->addArgument("--no-rtc");
	
	if ( KGlobal::config()->readBoolEntry( "agb", true ) )
		vba->addArgument("--agb-print");
	else
		vba->addArgument("--no-agb-print");
	
	if ( KGlobal::config()->readBoolEntry( "pwi", true ) ) {
		if ( vbaVersion >= 2 )
			vba->addArgument("--pause-when-inactive");
		else
			vba->addArgument("--pause-when-inative");
	} else
		vba->addArgument("--no-pause-when-inactive");
	
	switch ( KGlobal::config()->readNumEntry( "showspeed", 0 ) )
	{
		case 0: vba->addArgument("--no-show-speed"); break;
		case 1: vba->addArgument("--show-speed-normal"); break;
		case 2: vba->addArgument("--show-speed-detailed"); break;
	}
	
	switch ( KGlobal::config()->readNumEntry( "flashsize", 0 ) )
	{
		case 0: vba->addArgument("--flash-64k"); break;
		case 1: vba->addArgument("--flash-128k"); break;
	}
	
	if ( KGlobal::config()->readBoolEntry( "throttleenabled", false ) )
		vba->addArgument("--throttle=" + QString::number(KGlobal::config()->readNumEntry( "throttlevalue", 5 )));
	else
		vba->addArgument("--no-throttle");
	
	switch ( KGlobal::config()->readNumEntry( "savetype", 0 ) )
	{
		case 0: vba->addArgument("--save-auto"); break;
		case 1: vba->addArgument("--save-eeprom"); break;
		case 2: vba->addArgument("--save-sram"); break;
		case 3: vba->addArgument("--save-flash"); break;
		case 4: vba->addArgument("--save-sensor"); break;
	}
	
		KGlobal::config()->setGroup("graphics");
	switch ( KGlobal::config()->readNumEntry( "zoom", 0 ) )
	{
		case 0: vba->addArgument("--video-1x"); break;
		case 1: vba->addArgument("--video-2x"); break;
		case 2: vba->addArgument("--video-3x"); break;
		case 3: vba->addArgument("--video-4x"); break;
	}
	
	if ( KGlobal::config()->readBoolEntry( "afs", true ) )
		vba->addArgument("--auto-frameskip");
	else
		vba->addArgument("--no-auto-frameskip");

	if ( KGlobal::config()->readBoolEntry( "full", false ) )
		vba->addArgument("--fullscreen");
	
	vba->addArgument("--frameskip=" + QString::number(KGlobal::config()->readNumEntry( "frameskip" )));
	
	switch ( KGlobal::config()->readNumEntry( "filter", 0 ) )
	{
		case  0: vba->addArgument("--filter-normal"); break;
		case  1: vba->addArgument("--filter-tv-mode"); break;
		case  2: vba->addArgument("--filter-2xsai"); break;
		case  3: vba->addArgument("--filter-super-2xsai"); break;
		case  4: vba->addArgument("--filter-super-eagle"); break;
		case  5: vba->addArgument("--filter-pixelate"); break;
		case  6: vba->addArgument("--filter-motion-blur"); break;
		case  7: vba->addArgument("--filter-advmame"); break;
		case  8: vba->addArgument("--filter-simple2x"); break;
		case  9: vba->addArgument("--filter-bilinear"); break;
		case 10: vba->addArgument("--filter-bilinear+"); break;
		case 11: vba->addArgument("--filter-scanlines"); break;
	}
	
	if ( KGlobal::config()->readBoolEntry( "yuvenabled", false ) )
		vba->addArgument("--yuv=" + QString::number(KGlobal::config()->readNumEntry( "yuv" )));
	
	switch ( KGlobal::config()->readNumEntry( "ifb", 0 ) )
	{
		case 0: vba->addArgument("--ifb-none"); break;
		case 1: vba->addArgument("--ifb-motion-blur"); break;
		case 2: vba->addArgument("--ifb-smart"); break;
	}
}

void kvba::addFavorites()
{
	kvbaaddfavorites *addfav = new kvbaaddfavorites(this, "addfavorites");
	addfav->romPath->setURL(allDirs->currentURL().path());
	addfav->exec();
	
	delete addfav;
}

void kvba::favDoubleClicked(QListViewItem *i)
{
	vba->clearArguments();
	KGlobal::config()->setGroup("general");
	vba->addArgument(KGlobal::config()->readEntry("exepath"));

	if ( ! i->text(2).isEmpty() )
		vba->addArgument("--ips=" + i->text(2));

	if ( ! i->text(3).isEmpty() )
		vba->addArgument("--config=" + i->text(3));
	else
		vbaPrepareArguments();
		

	vba->addArgument(i->text(1));
	
	vbaOutput->clear();
	vba->start();
	
	romsView->setCurrentPage(2);
}

void kvba::fileNewConfigActivated()
{
	kvbajoyconf *joyconf = new kvbajoyconf(this, "joyconf");
	joyconf->exec();
	delete joyconf;
}

void kvba::favChangeSelection()
{
	editFavoritesAction->setEnabled(true);
}

void kvba::editFavorites()
{
	kvbaeditfavorites *editfav = new kvbaeditfavorites(this, "editfavorites");
	editfav->gameName->setText(favorites->currentItem()->text(0));
	editfav->romPath->setURL(favorites->currentItem()->text(1));
	editfav->ipsPath->setURL(favorites->currentItem()->text(2));
	editfav->cfgPath->setURL(favorites->currentItem()->text(3));
	editfav->favnum = favorites->itemPos(favorites->currentItem());
	editfav->exec();
	
	delete editfav;
}

#include "kvba.moc"
